/////////////////////////////////////////////////////////////////////////////
//// INCLUDE FILES

#include "SNTRANS.h"

// Copyright And Configuration Management ----------------------------------
//
// Stubs For NDIS 3.X PROTOCOL Functions For SNTRANS Example - TransFcn.c
//
//    Copyright (c) 1995-1997, Printing Communications Associates, Inc.
//
//                             Thomas F. Divine
//                           4201 Brunswick Court
//                        Smyrna, Georgia 30080 USA
//                              (770) 436-1714
//
// $Workfile$
//  $Logfile$
//   $Author: FRED $
//     $Date: 1995/11/28 08:35:59 $
// $Revision: 1.1 $
//
// $Log: transfcn.c $
// Revision 1.1  1995/11/28 08:35:59  FRED
// Initial revision
// 
// End ---------------------------------------------------------------------

// TRANSFCN.C - stubs for NDIS PROTOCOL functions for SNTRANS example


/////////////////////////////////////////////////////////////////////////////
//// GLOBAL DATA

PROTOCOL_BLOCK SNTRANSContext={0};


/////////////////////////////////////////////////////////////////////////////
//// NDIS 3.1 PROTOCOL FUNCTION STUBS

VOID NDIS_API
SNTRANSOpenAdapterCompleteHandler(
    IN NDIS_HANDLE NdisBindingContext,
    IN NDIS_STATUS Status,
    IN NDIS_STATUS OpenErrorStatus
    )
{
	PPROTOCOL_BLOCK pSNTRANSContext;

#ifdef DEBUG
	dprintf( "OpenAdapterCompleteHandler Status 0x%x\n", Status );
#endif // DEBUG

	/* Sanity Checks On Arguments
	----------------------------- */
	Assert( NdisBindingContext );

	if( !NdisBindingContext )
	{
		return;
	}

	/* Get Driver Context
	--------------------- */
	pSNTRANSContext = (PPROTOCOL_BLOCK )NdisBindingContext;

	/* Check Open Adapter Results
	----------------------------- */
	ASSERT( Status == NDIS_STATUS_SUCCESS );

	if( Status != NDIS_STATUS_SUCCESS )
	{
		// TODO: Handle OpenAdapter failure

		return;
	}

	// TODO: Handle OpenAdapter success
}

VOID NDIS_API
SNTRANSCloseAdapterCompleteHandler(
    IN NDIS_HANDLE NdisBindingContext,
    IN NDIS_STATUS Status
    )
{
	DEBUGTRACE( "CloseAdapterCompleteHandler Entry..." );
}

VOID NDIS_API
SNTRANSSendCompleteHandler(
    IN NDIS_HANDLE NdisBindingContext,
    IN PNDIS_PACKET Packet,
    IN NDIS_STATUS Status
    )
{
	DEBUGTRACE( "SendCompleteHandler Entry..." );
}

VOID NDIS_API
SNTRANSTransferDataCompleteHandler(
    IN NDIS_HANDLE NdisBindingContext,
    IN PNDIS_PACKET Packet,
    IN NDIS_STATUS Status,
    IN UINT BytesTransferred
    )
{
	DEBUGTRACE( "TransferDataCompleteHandler Entry..." );
}

VOID NDIS_API
SNTRANSResetCompleteHandler(
    IN NDIS_HANDLE NdisBindingContext,
    IN NDIS_STATUS Status
    )
{
	DEBUGTRACE( "ResetCompleteHandler Entry..." );
}

VOID NDIS_API
SNTRANSRequestCompleteHandler(
    IN NDIS_HANDLE NdisBindingContext,
    IN PNDIS_REQUEST NdisRequest,
    IN NDIS_STATUS Status
    )
{
	DEBUGTRACE( "RequestCompleteHandler Entry..." );
}

NDIS_STATUS NDIS_API
SNTRANSReceiveHandler(
    IN NDIS_HANDLE NdisBindingContext,
    IN NDIS_HANDLE MacReceiveContext,
    IN PVOID HeaderBuffer,
    IN UINT HeaderBufferSize,
    IN PVOID LookAheadBuffer,
    IN UINT LookaheadBufferSize,
    IN UINT PacketSize
    )
{
	DEBUGTRACE( "ReceiveHandler Entry..." );

	return( NDIS_STATUS_SUCCESS );
}

VOID NDIS_API
SNTRANSReceiveCompleteHandler(
    IN NDIS_HANDLE NdisBindingContext
    )
{
	DEBUGTRACE( "ReceiveCompleteHandler Entry..." );
}

VOID NDIS_API
SNTRANSStatusHandler(
    IN NDIS_HANDLE NdisBindingContext,
    IN NDIS_STATUS GeneralStatus,
    IN PVOID StatusBuffer,
    IN UINT StatusBufferSize
    )
{
	DEBUGTRACE( "StatusHandler Entry..." );
}

VOID NDIS_API
SNTRANSStatusCompleteHandler(
    IN NDIS_HANDLE NdisBindingContext
    )
{
	DEBUGTRACE( "StatusCompleteHandler Entry..." );
}


//
// BIND_ADAPTER_HANDLER, UNBIND_ADAPTER_HANDLER, UNLOAD_PROTOCOL
// are valid for NDIS 3.1 protocols
//

VOID NDIS_API
SNTRANSBindAdapterHandler(
    OUT PNDIS_STATUS    pProtocolBindStatus,
    IN  NDIS_HANDLE     BindAdapterContext,
    IN  PNDIS_STRING    AdapterName,
    IN  PVOID           SystemSpecific1,
    IN  PVOID           SystemSpecific2
    )
{
	DEBUGTRACE( "BindAdapterHandler Entry..." );

	*pProtocolBindStatus = NDIS_STATUS_PENDING;

	/* Assume Failure
	----------------- */
	SNTRANSContext.nOpenAdapterStatus = NDIS_STATUS_FAILURE;
	SNTRANSContext.nOpenAdapterErrorStatus = NDIS_STATUS_FAILURE;

	/* Specify Desired Medium
	------------------------- */
	SNTRANSContext.nMedium;
	SNTRANSContext.MediumArray=NdisMedium802_3;

	/* Save The BindAdapterContext
	------------------------------ */
	SNTRANSContext.hBindAdapterContext = BindAdapterContext;

	//
	//  Try to open the MAC
	//

	DEBUGTRACE( "Calling NdisOpenAdapter..." );

	NdisOpenAdapter(
		&SNTRANSContext.nOpenAdapterStatus,
		&SNTRANSContext.nOpenAdapterErrorStatus,
		&SNTRANSContext.hNdisAdapterHandle,
		&SNTRANSContext.nMedium,
		&SNTRANSContext.MediumArray,
		1,
		SNTRANSContext.hNdisProtocolHandle,
		(NDIS_HANDLE )&SNTRANSContext,		// Specify ProtocolBindingContext
		AdapterName,
		0,
		NULL
		);

	/* ATTENTION!!!
	---------------
	 * The following lines of code handling the NDIS_STATUS_PENDING return
	 * from NdisOpenAdapter and the return value from this function have
	 * been proven to work on a wide variety of MAC adapters. However, it
	 * may need some refinement in the future.
	 *
	 * Specifically, this function always reports NDIS_STATUS_SUCCESS - even
	 * in the case that the NdisOpenAdapter call returns NDIS_STATUS_PENDING.
	 * A better (possibly) approach would be for this function to return
	 * NDIS_STATUS_PENDING instead of NDIS_STATUS_SUCCESS if NdisOpenAdapter
	 * returned NDIS_STATUS_PENDING; if this change was made, then
	 * NdisBindAdapterComplete would need to be called from
	 * LOWEROpenAdapterCompleteHandler.
	 */

	if( SNTRANSContext.nOpenAdapterStatus != NDIS_STATUS_PENDING )
	{
		SNTRANSOpenAdapterCompleteHandler(
			(NDIS_HANDLE )&SNTRANSContext,	// ProtocolBindingContext
			SNTRANSContext.nOpenAdapterStatus,
			SNTRANSContext.nOpenAdapterErrorStatus
			);
	}

	*pProtocolBindStatus = NDIS_STATUS_SUCCESS;		// Report Success

	DEBUGTRACE( "BindAdapterHandler Exit\n" );
}


/* Note
-------
 * It appears that UnloadProtocolHandler() is not called if this function
 * returns a value other than NDIS_STATUS_SUCCESS.
 */
VOID NDIS_API
SNTRANSUnbindAdapterHandler(
    OUT PNDIS_STATUS    pStatus,
    IN  NDIS_HANDLE     ProtocolBindingContext,
    IN  NDIS_HANDLE     UnbindAdapterContext
    )
{
	DEBUGTRACE( "UnbindAdapterHandler Entry..." );

	NdisCloseAdapter(
		pStatus,
		SNTRANSContext.hNdisAdapterHandle		// Handle From NdisOpenAdapter
		);

	if( *pStatus != NDIS_STATUS_PENDING )
	{
		SNTRANSCloseAdapterCompleteHandler(
			ProtocolBindingContext,
			*pStatus
			);
	}
}

VOID NDIS_API
SNTRANSUnloadProtocolHandler( void )
{
	NDIS_STATUS Status = NDIS_STATUS_FAILURE;

	DEBUGTRACE( "UnloadProtocolHandler Entry..." );

	NdisDeregisterProtocol(
		&Status,
		SNTRANSContext.hNdisProtocolHandle	// Handle From NdisRegisterProtocol
		);
}
